@@ -9,6 +9,12 @@ pub use self::sys::types::{Net, Node, State};
 
 use self::sys::functions::*;
 
+use std::{
+    fs::{self, File},
+    io::{Read, Write},
+    path::Path,
+};
+
 /// A wrapper type for net to support managing multiple states for simulations.
 pub struct SimulatedNet {
     net: Net,
@@ -129,7 +135,7 @@ pub enum DynamicNet {
 /// A trait to share safe functionality for the default `Net` type and the `SimulatedNet`
 pub trait SafeNet {
     /// Save the petri net to a file. Result represents success.
-    fn save(&self, filename: &str) -> bool;
+    fn save(&self, filename: &Path) -> bool;
     /// A slice of nodes representing the transitions of the petri net.
     fn transitions(&self) -> &[Node];
     /// Returns the index of the next reusable transition.
@@ -231,7 +237,7 @@ impl SimulatedNet {
         }
     }
     /// Load a petri net from a file in simulated state.
-    pub fn load(filename: &str) -> Option<SimulatedNet> {
+    pub fn load(filename: &Path) -> Option<SimulatedNet> {
         Some(SimulatedNet {
             net: Net::load(filename)?,
             states: Vec::new(),
@@ -257,7 +263,7 @@ impl SimulatedNet {
     /// Load a simulation state from a file.
     /// Using the wrong file layout will fail or create confusing state.
     #[must_use]
-    pub fn load_state(&mut self, filename: &str) -> Option<()> {
+    pub fn load_state(&mut self, filename: &Path) -> Option<()> {
         if let Some(state) = State::load(&self.net, filename) {
             self.states.push(state);
             Some(())
@@ -306,7 +312,7 @@ impl SimulatedNet {
 
 impl SafeNet for SimulatedNet {
     #[inline]
-    fn save(&self, filename: &str) -> bool {
+    fn save(&self, filename: &Path) -> bool {
         self.net.save(filename)
     }
 
@@ -394,8 +400,6 @@ impl SafeNet for SimulatedNet {
     }
 }
 
-use std::ffi::CString;
-
 impl Node {
     /// Get a slice of indices to the next nodes.
     pub fn next(&self) -> &[u32] {
@@ -418,11 +422,25 @@ impl Net {
         }
     }
     /// Load a petri net from a file.
-    pub fn load(filename: &str) -> Option<Net> {
-        let path = CString::new(filename).unwrap();
+    pub fn load(filename: &Path) -> Option<Net> {
+        let size = (fs::metadata(filename).ok()?.len() + 3) / 4;
+
+        let mut values;
+        {
+            let mut file = File::open(filename).ok()?;
+
+            values = vec![0; size as usize * 4];
+            file.read_exact(&mut values).ok()?;
+        }
+
         unsafe {
             let mut net = std::mem::MaybeUninit::uninit();
-            if pnsLoadNet(net.as_mut_ptr(), path.as_ptr()) {
+
+            if pnsLoadNet(
+                net.as_mut_ptr(),
+                size as u32,
+                std::mem::transmute::<_, *mut u32>(&values[0]),
+            ) {
                 Some(net.assume_init())
             } else {
                 None
@@ -450,9 +468,23 @@ impl Net {
 }
 
 impl SafeNet for Net {
-    fn save(&self, filename: &str) -> bool {
-        let path = CString::new(filename).unwrap();
-        unsafe { pnsNet_save(self, path.as_ptr()) }
+    fn save(&self, filename: &Path) -> bool {
+        let mut file = File::create(filename).unwrap();
+
+        let data = unsafe {
+            let count = pnsNet_serializeSize(self);
+            let mut data = vec![0u32; count as usize];
+            pnsNet_serialize(self, &mut data[0]);
+            data
+        };
+
+        for value in data {
+            if file.write_all(&value.to_ne_bytes()).is_err() {
+                return false;
+            }
+        }
+
+        true
     }
 
     fn transitions(&self) -> &[Node] {
@@ -553,11 +585,25 @@ impl State {
         }
     }
 
-    fn load(net: &Net, filename: &str) -> Option<State> {
-        let path = CString::new(filename).unwrap();
+    fn load(net: &Net, filename: &Path) -> Option<State> {
+        let size = (fs::metadata(filename).ok()?.len() + 3) / 4;
+
+        let mut values;
+        {
+            let mut file = File::open(filename).ok()?;
+
+            values = vec![0; size as usize * 4];
+            file.read_exact(&mut values).ok()?;
+        }
+
         unsafe {
             let mut state = std::mem::MaybeUninit::uninit();
-            if pnsLoadState(state.as_mut_ptr(), net, path.as_ptr()) {
+            if pnsLoadState(
+                state.as_mut_ptr(),
+                net,
+                size as u32,
+                std::mem::transmute::<_, *mut u32>(&mut values[0]),
+            ) {
                 Some(state.assume_init())
             } else {
                 None
@@ -615,9 +661,15 @@ impl Drop for State {
 
 impl<'net> SimulationState<'net> {
     /// Save the current state of the simulation.
-    pub fn save(&self, filename: &str) -> bool {
-        let path = CString::new(filename).unwrap();
-        unsafe { pnsState_save(self.state, self.net, path.as_ptr()) }
+    pub fn save(&self, filename: &Path) -> bool {
+        let mut file = File::create(filename).unwrap();
+        for count in self.call_counts() {
+            if file.write_all(&count.to_ne_bytes()).is_err() {
+                return false;
+            }
+        }
+
+        true
     }
 
     /// List the current call counts of all transitions.
@@ -638,9 +690,15 @@ impl<'net> SimulationState<'net> {
 
 impl<'net> SimulationStateMut<'net> {
     /// Save the current state of the simulation.
-    pub fn save(&self, filename: &str) -> bool {
-        let path = CString::new(filename).unwrap();
-        unsafe { pnsState_save(self.state, self.net, path.as_ptr()) }
+    pub fn save(&self, filename: &Path) -> bool {
+        let mut file = File::create(filename).unwrap();
+        for count in self.call_counts() {
+            if file.write_all(&count.to_ne_bytes()).is_err() {
+                return false;
+            }
+        }
+
+        true
     }
 
     /// List the current call counts of all transitions.
